//
//  PUMainViewController.m
//  BJHL-VideoPlayer-UI
//
//  Created by DLM on 2017/4/26.
//  Copyright © 2017年 杨健. All rights reserved.
//

#import "PUMainViewController.h"

#import <BJPlayerManagerUI/BJPlayerManagerUI.h>
#import <BJPlayerManagerUI/BJPUMacro.h>
#import <BJPlayerManagerCore/BJPlayerManagerCore.h>
#import <Masonry/Masonry.h>

#import "UIAlertView+bjp.h"
#import "PUPlayOptionsViewController.h"
#import "PUPlayViewController.h"
#import "BJPViewController.h"
#import "PUPMDownloadViewController.h"

@interface PUMainViewController ()
@property (nonatomic) UISegmentedControl *deploySegment;
@property (nonatomic) UITextField *vidTextField;
@property (nonatomic) UITextField *tokenTextField;

@property (nonatomic) UIButton *settingButton;
@property (nonatomic) UIButton *avPlayButton;
@property (nonatomic) UIButton *downloadBtn;
@property (nonatomic) UIButton *ijkPlayButton;
@property (nonatomic) UIButton *playbackButton;

@property (nonatomic) UIScrollView *scrollView;
@property (nonatomic) UIView *contentView;

@end

@implementation PUMainViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.view.backgroundColor = [UIColor whiteColor];
    
    [self setupScrollView];
    
#if DEBUG
    [self.contentView addSubview:self.deploySegment];
    [self deploySegmentMakeConstraints];
#else
#endif
    [self.contentView addSubview:self.vidTextField];
    [self.contentView addSubview:self.tokenTextField];
    
    [self.contentView addSubview:self.settingButton];
    [self.contentView addSubview:self.avPlayButton];
    [self.contentView addSubview:self.downloadBtn];
    [self.contentView addSubview:self.ijkVideoPlayButton];
    [self.contentView addSubview:self.playbackButton];
    
    [self makeConstraints];
    
    self.vidTextField.text = @"12057983";
    self.tokenTextField.text = @"IfMH65F6Vs4-eHlkNx1Yl2SDt1EOmiUXLgznNiwRBh4LjSMBaGipUg";
}

- (void)setupScrollView {
    self.contentView = [UIView new];
    self.scrollView = [UIScrollView new];
    
    self.scrollView.contentSize = CGSizeMake(self.view.bounds.size.width, self.view.frame.size.height);
    
    [self.view addSubview:self.scrollView];
    [self.scrollView addSubview:self.contentView];
}

- (void)deploySegmentMakeConstraints {
    [self.deploySegment mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.equalTo(self.contentView).offset(20.f);
        make.right.equalTo(self.contentView).offset(-20.f);
        make.height.equalTo(@30);
    }];
}

- (void)makeConstraints {
    
    [self.scrollView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.view);
    }];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.scrollView);
        make.width.equalTo(self.view);
    }];
    
    [self.vidTextField mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.equalTo(@30);
        make.left.equalTo(self.contentView).offset(30);
        make.right.equalTo(self.contentView).offset(-30);
        make.top.equalTo(self.contentView).offset(70);
    }];
    
    [self.tokenTextField mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.equalTo(@30);
        make.left.right.equalTo(self.vidTextField);
        make.top.equalTo(self.vidTextField.mas_bottom).offset(30);
    }];
    
    [self.settingButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.tokenTextField.mas_bottom).offset(30.0);
        make.left.right.equalTo(self.vidTextField);
        make.height.equalTo(@30);
    }];
    
    [self.avPlayButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.equalTo(@30);
        make.left.right.equalTo(self.vidTextField);
        make.top.equalTo(self.settingButton.mas_bottom).offset(30);
    }];
    
    [self.downloadBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.equalTo(@30);
        make.left.right.equalTo(self.vidTextField);
        make.top.equalTo(self.avPlayButton.mas_bottom).offset(30);
    }];
    
    [self.ijkVideoPlayButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.equalTo(@30);
        make.left.right.equalTo(self.vidTextField);
        make.top.equalTo(self.downloadBtn.mas_bottom).offset(30);
    }];
    
    [self.playbackButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.equalTo(@30);
        make.left.right.equalTo(self.vidTextField);
        make.top.equalTo(self.ijkVideoPlayButton.mas_bottom).offset(30);
        make.bottom.equalTo(self.contentView.mas_bottom).offset(-30);
    }];
}

#pragma mark - action

- (void)showSettingView {
    PUPlayOptionsViewController *optionsVC = [[PUPlayOptionsViewController alloc] init];
    [self presentViewController:optionsVC animated:YES completion:nil];
}

- (void)entryPlayControl {
    NSString *vid = self.vidTextField.text;
    NSString *token = self.tokenTextField.text;
    PUPlayViewController *playerVC = [[PUPlayViewController alloc] initWithVid:vid
                                                                         token:token
                                                                    playerType:PUPlayerType_AVPlayer];
    [self.navigationController pushViewController:playerVC animated:YES];
}

- (void)downloadAction {
    PUPMDownloadViewController *vc = [PUPMDownloadViewController new];
    [self.navigationController pushViewController:vc animated:YES];
}

- (void)deployTypeChanged:(UISegmentedControl *)segment {
#if DEBUG
    switch (segment.selectedSegmentIndex) {
        case 0:
            [PMAppConfig sharedInstance].deployType = PMDeployType_www;
            [self saveDeplayAndTipWithEnvType:0];
            break;
        case 1:
            [PMAppConfig sharedInstance].deployType = PMDeployType_beta;
            [self saveDeplayAndTipWithEnvType:1];
            break;
        case 2:
            [PMAppConfig sharedInstance].deployType = PMDeployType_test;
            [self saveDeplayAndTipWithEnvType:2];
            break;
        default:
            break;
    }
#else
#endif
}

- (void)enterIJKPlay {
    NSString *vid = self.vidTextField.text;
    NSString *token = self.tokenTextField.text;
    PUPlayViewController *playerVC = [[PUPlayViewController alloc]
                                      initWithVid:vid
                                      token:token
                                      playerType:PUPlayerType_IJKPlayer];
    [self.navigationController pushViewController:playerVC animated:YES];
}

- (void)playbackAction {
    [self.navigationController pushViewController:[BJPViewController new] animated:YES];
}


#pragma MARK - 切换环境
- (void)saveDeplayAndTipWithEnvType:(PMDeployType)type {
    [self saveDeplay:type];
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"切换成功" message:@"必须重新启动app设置才生效" delegate:nil cancelButtonTitle:nil otherButtonTitles:@"退出程序", nil];
    [alert showAlertViewWithCompleteBlock:^(NSInteger buttonIndex) {
        abort();
    }];
}

- (void)saveDeplay:(PMDeployType)type {
    [[NSUserDefaults standardUserDefaults] setInteger:type forKey:@"developType"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)playback {
    BJPViewController *vc = [[BJPViewController alloc] init];
    [self.navigationController pushViewController:vc animated:YES];
}

#pragma mark - set get

- (UISegmentedControl *)deploySegment
{
    if (!_deploySegment) {
        _deploySegment = [[UISegmentedControl alloc] initWithItems:@[@"www", @"Beta", @"test"]];
        NSInteger index = [[NSUserDefaults standardUserDefaults] integerForKey:@"developType"];
        _deploySegment.selectedSegmentIndex = index;
        [self setDevelopType:index];
        [_deploySegment addTarget:self action:@selector(deployTypeChanged:) forControlEvents:UIControlEventValueChanged];
    }
    return _deploySegment;
}

- (UITextField *)vidTextField
{
    if(!_vidTextField) {
        _vidTextField = [[UITextField alloc] init];
        _vidTextField.placeholder = @"vid";
        _vidTextField.layer.borderColor = [UIColor grayColor].CGColor;
        _vidTextField.layer.borderWidth = 0.5;
        _vidTextField.layer.cornerRadius = 5.f;
    }
    return _vidTextField;
}

- (UITextField *)tokenTextField
{
    if(!_tokenTextField) {
        _tokenTextField = [[UITextField alloc] init];
        _tokenTextField.placeholder = @"token";
        _tokenTextField.text = @"test12345678";//测试环境
        _tokenTextField.layer.borderColor = [UIColor grayColor].CGColor;
        _tokenTextField.layer.borderWidth = 0.5;
        _tokenTextField.layer.cornerRadius = 5.f;
    }
    return _tokenTextField;
}

- (UIButton *)settingButton {
    if (!_settingButton) {
        _settingButton = [[UIButton alloc] init];
        _settingButton.backgroundColor = [UIColor redColor];
        _settingButton.layer.cornerRadius = 5.f;
        [_settingButton setTitle:@"播放设置" forState:UIControlStateNormal];
        [_settingButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_settingButton.titleLabel setFont:[UIFont systemFontOfSize:15.f]];
        [_settingButton addTarget:self action:@selector(showSettingView) forControlEvents:UIControlEventTouchUpInside];
    }
    return _settingButton;
}

- (UIButton *)avPlayButton
{
    if (!_avPlayButton) {
        _avPlayButton = [[UIButton alloc] init];
        _avPlayButton.backgroundColor = [UIColor redColor];
        _avPlayButton.layer.cornerRadius = 5.f;
        [_avPlayButton setTitle:@"avplayer播放" forState:UIControlStateNormal];
        [_avPlayButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_avPlayButton.titleLabel setFont:[UIFont systemFontOfSize:15.f]];
        [_avPlayButton addTarget:self action:@selector(entryPlayControl) forControlEvents:UIControlEventTouchUpInside];
    }
    return _avPlayButton;
}

- (UIButton *)downloadBtn
{
    if (!_downloadBtn) {
        _downloadBtn = [[UIButton alloc] init];
        _downloadBtn.backgroundColor = [UIColor redColor];
        _downloadBtn.layer.cornerRadius = 5.f;
        [_downloadBtn setTitle:@"下载" forState:UIControlStateNormal];
        [_downloadBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_downloadBtn.titleLabel setFont:[UIFont systemFontOfSize:15.f]];
        [_downloadBtn addTarget:self action:@selector(downloadAction) forControlEvents:UIControlEventTouchUpInside];
    }
    return _downloadBtn;
}

- (UIButton *)ijkVideoPlayButton {
    if(!_ijkPlayButton) {
        _ijkPlayButton = [[UIButton alloc] init];
        _ijkPlayButton.backgroundColor = [UIColor redColor];
        _ijkPlayButton.layer.cornerRadius = 5.f;
        [_ijkPlayButton setTitle:@"ijkplayer播放" forState:UIControlStateNormal];
        [_ijkPlayButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_ijkPlayButton.titleLabel setFont:[UIFont systemFontOfSize:15.f]];
        [_ijkPlayButton addTarget:self action:@selector(enterIJKPlay) forControlEvents:UIControlEventTouchUpInside];
    }
    return _ijkPlayButton;
}

- (UIButton *)playbackButton {
    if(!_playbackButton) {
        _playbackButton = [[UIButton alloc] init];
        _playbackButton.backgroundColor = [UIColor redColor];
        _playbackButton.layer.cornerRadius = 5.f;
        [_playbackButton setTitle:@"回放播放" forState:UIControlStateNormal];
        [_playbackButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_playbackButton.titleLabel setFont:[UIFont systemFontOfSize:15.f]];
        [_playbackButton addTarget:self action:@selector(playback) forControlEvents:UIControlEventTouchUpInside];
    }
    return _playbackButton;
}

#pragma MARK - 切换环境

- (void)setDevelopType:(NSInteger)type {
    
#if DEBUG
    switch (type) {
        case 0:
            [PMAppConfig sharedInstance].deployType = PMDeployType_www;
            break;
            
        case 1:
            [PMAppConfig sharedInstance].deployType = PMDeployType_beta;
            break;
            
        case 2:
            [PMAppConfig sharedInstance].deployType = PMDeployType_test;
            break;
            
        default:
            break;
    }
#else
#endif
}

@end
