//
//  PUPlayOptionsViewController.h
//  BJPlayerManagerUI_Example
//
//  Created by daixijia on 2018/8/2.
//  Copyright © 2018年 oushizishu. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <BJLiveBase/NSObject+BJLYYModel.h>

NS_ASSUME_NONNULL_BEGIN

@interface PUPlayOptions : NSObject <BJLYYModel>

@property (nonatomic) BOOL advertisementEnabled;        // 播放片头片尾
@property (nonatomic) BOOL notAutoStartImageAD;         // 不自动开始图片广告
@property (nonatomic) BOOL autoplay;                    // 自动播放
@property (nonatomic) BOOL sliderDragEnabled;           // 能否拖动进度条
@property (nonatomic) BOOL playTimeRecordEnabled;       // 开启记忆播放
@property (nonatomic) BOOL encryptEnabled;              // 开启加密
@property (nonatomic) BOOL backgroundAudioEnabled;      // 开启后台播放
@property (nonatomic, nullable) NSString *exclusiveDomain;        // 专属域名

/**
 第三方鉴权参数
 视频如果需要请求第三方服务器查看是否有权限获取数据，就需要设置这个属性
 没有通过鉴权，或者没有鉴权，获取播放信息的回调，设置视频播放的回调，播放器的代理将会返回错误 BJPMErrorCodeAccessKeyDeny
 */
@property (nonatomic, nullable) NSString *accessKey;

/** 初始播放时间，如果开启了记忆播放则设置无效 */
@property (nonatomic) NSTimeInterval initialPlaybackTime;

/**
 播放倍速
 
 #discussion 设置播放倍速，如果设置了这个值，之后所有的视频都通过这个倍速来控制
 #discussion 如果在播放视频的时候改变了倍速，将记住该倍速
 */
@property (nonatomic) CGFloat playRate;

/**
 偏好清晰度列表
 
 #discussion 优先使用此列表中的清晰度播放在线视频，优先级按数组元素顺序递减
 #discussion 列表元素为 PMVideoDefinitionType 转换成的 NSNumber， 如 @[@(DT_HIGH)]
 #discussion 此设置对播放视频无效，仅用于下载
 */
@property (nonatomic, nullable) NSArray<NSNumber *> *preferredDefinitionList;

/**
 第三方用户名和编号
 
 #discussion 用于上报统计
 */
@property (nonatomic, nullable) NSString *userName;
@property (nonatomic) NSInteger userNumber;

+ (instancetype)shareInstance;

@end

@interface PUPlayOptionsViewController : UIViewController

@end

NS_ASSUME_NONNULL_END
